@@ -42,10 +42,18 @@ class DiskFileError(SwiftException):
     pass
 
 
+class DiskFileCollision(SwiftException):
+    pass
+
+
 class DiskFileNotExist(SwiftException):
     pass
 
 
+class DiskFileNoSpace(SwiftException):
+    pass
+
+
 class PathNotDir(OSError):
     pass
 
