@@ -27,6 +27,8 @@ maximum lookup depth. If a match is found, the environment's Host header is
 rewritten and the request is passed further down the WSGI chain.
 """
 
+import socket
+
 try:
     import dns.resolver
     from dns.exception import DNSException
@@ -58,6 +60,18 @@ def lookup_cname(domain):  # pragma: no cover
         return 0, None
 
 
+def is_ip(domain):
+    try:
+        socket.inet_pton(socket.AF_INET, domain)
+        return True
+    except socket.error:
+        try:
+            socket.inet_pton(socket.AF_INET6, domain)
+            return True
+        except socket.error:
+            return False
+
+
 class CNAMELookupMiddleware(object):
     """
     CNAME Lookup Middleware
@@ -93,6 +107,8 @@ class CNAMELookupMiddleware(object):
             given_domain, port = given_domain.rsplit(':', 1)
         if given_domain == self.storage_domain[1:]:  # strip initial '.'
             return self.app(env, start_response)
+        if is_ip(given_domain):
+            return self.app(env, start_response)
         a_domain = given_domain
         if not a_domain.endswith(self.storage_domain):
             if self.memcache is None:
