@@ -23,7 +23,7 @@ The logging format implemented below is as follows:
 
 client_ip remote_addr datetime request_method request_path protocol
     status_int referer user_agent auth_token bytes_recvd bytes_sent
-    client_etag transaction_id headers request_time source
+    client_etag transaction_id headers request_time source log_info
 
 These values are space-separated, and each is url-encoded, so that they can
 be separated with a simple .split()
@@ -33,6 +33,18 @@ be separated with a simple .split()
   from the X-Forwarded-For header, X-Cluster-Ip header, or the REMOTE_ADDR
   environment variable.
 
+* source (swift.source in the WSGI environment) indicates the code
+  that generated the request, such as most middleware. (See below for
+  more detail.)
+
+* log_info (swift.log_info in the WSGI environment) is for additional
+  information that could prove quite useful, such as any x-delete-at
+  value or other "behind the scenes" activity that might not
+  otherwise be detectable from the plain log information. Code that
+  wishes to add additional log information should use code like
+  ``env.setdefault('swift.log_info', []).append(your_info)`` so as to
+  not disturb others' log information.
+
 * Values that are missing (e.g. due to a header not being present) or zero
   are generally represented by a single hyphen ('-').
 
@@ -66,6 +78,8 @@ from swift.common.utils import (get_logger, get_remote_client,
                                 get_valid_utf8_str, config_true_value,
                                 InputProxy)
 
+QUOTE_SAFE = '/:'
+
 
 class ProxyLoggingMiddleware(object):
     """
@@ -122,7 +136,7 @@ class ProxyLoggingMiddleware(object):
         if self.req_already_logged(req):
             return
         req_path = get_valid_utf8_str(req.path)
-        the_request = quote(unquote(req_path))
+        the_request = quote(unquote(req_path), QUOTE_SAFE)
         if req.query_string:
             the_request = the_request + '?' + req.query_string
         logged_headers = None
@@ -131,7 +145,7 @@ class ProxyLoggingMiddleware(object):
                                        for k, v in req.headers.items())
         method = self.method_from_req(req)
         self.access_logger.info(' '.join(
-            quote(str(x) if x else '-')
+            quote(str(x) if x else '-', QUOTE_SAFE)
             for x in (
                 get_remote_client(req),
                 req.remote_addr,
@@ -150,6 +164,7 @@ class ProxyLoggingMiddleware(object):
                 logged_headers,
                 '%.4f' % request_time,
                 req.environ.get('swift.source'),
+                ','.join(req.environ.get('swift.log_info') or '-'),
             )))
         self.mark_req_logged(req)
         # Log timing and bytes-transfered data to StatsD
