@@ -35,8 +35,6 @@ added. For example::
 
     [filter:staticweb]
     use = egg:swift#staticweb
-    # Seconds to cache container x-container-meta-web-* header values.
-    # cache_timeout = 300
 
 Any publicly readable containers (for example, ``X-Container-Read: .r:*``, see
 `acls`_ for more information on this) will be checked for
@@ -67,6 +65,12 @@ the .../listing.css style sheet. If you "view source" in your browser on a
 listing page, you will see the well defined document structure that can be
 styled.
 
+The content-type of directory marker objects can be modified by setting
+the ``X-Container-Meta-Web-Directory-Type`` header.  If the header is not set,
+application/directory is used by default.  Directory marker objects are
+0-byte objects that represent directories to create a simulated hierarchical
+structure.
+
 Example usage of this middleware via ``swift``:
 
     Make the container publicly readable::
@@ -99,20 +103,26 @@ Example usage of this middleware via ``swift``:
         swift post -m 'web-error:error.html' container
 
     Now 401's should load 401error.html, 404's should load 404error.html, etc.
+
+    Set Content-Type of directory marker object::
+
+        swift post -m 'web-directory-type:text/directory' container
+
+    Now 0-byte objects with a content-type of text/directory will be treated
+    as directories rather than objects.
 """
 
 
 import cgi
 import time
-from urllib import unquote, quote as urllib_quote
+from urllib import quote as urllib_quote
 
-
-from swift.common.utils import cache_from_env, human_readable, split_path, \
-    config_true_value, json
-from swift.common.wsgi import make_pre_authed_env, make_pre_authed_request, \
-    WSGIContext
+from swift.common.utils import human_readable, split_path, config_true_value, \
+    json
+from swift.common.wsgi import make_pre_authed_env, WSGIContext
 from swift.common.http import is_success, is_redirection, HTTP_NOT_FOUND
 from swift.common.swob import Response, HTTPMovedPermanently, HTTPNotFound
+from swift.proxy.controllers.base import get_container_info
 
 
 def quote(value, safe='/'):
@@ -145,7 +155,8 @@ class _StaticWebContext(WSGIContext):
         self.cache_timeout = staticweb.cache_timeout
         self.agent = '%(orig)s StaticWeb'
         # Results from the last call to self._get_container_info.
-        self._index = self._error = self._listings = self._listings_css = None
+        self._index = self._error = self._listings = self._listings_css = \
+            self._dir_type = None
 
     def _error_response(self, response, env, start_response):
         """
@@ -179,41 +190,23 @@ class _StaticWebContext(WSGIContext):
     def _get_container_info(self, env):
         """
         Retrieves x-container-meta-web-index, x-container-meta-web-error,
-        x-container-meta-web-listings, and x-container-meta-web-listings-css
-        from memcache or from the cluster and stores the result in memcache and
-        in self._index, self._error, self._listings, and self._listings_css.
+        x-container-meta-web-listings, x-container-meta-web-listings-css,
+        and x-container-meta-web-directory-type from memcache or from the
+        cluster and stores the result in memcache and in self._index,
+        self._error, self._listings, self._listings_css and self._dir_type.
 
         :param env: The WSGI environment dict.
         """
-        self._index = self._error = self._listings = self._listings_css = None
-        memcache_client = cache_from_env(env)
-        if memcache_client:
-            memcache_key = '/staticweb/%s/%s/%s' % (self.version, self.account,
-                                                    self.container)
-            cached_data = memcache_client.get(memcache_key)
-            if cached_data:
-                (self._index, self._error, self._listings,
-                 self._listings_css) = cached_data
-                return
-        resp = make_pre_authed_request(
-            env, 'HEAD', '/%s/%s/%s' % (
-                self.version, self.account, self.container),
-            agent=self.agent, swift_source='SW').get_response(self.app)
-        if is_success(resp.status_int):
-            self._index = \
-                resp.headers.get('x-container-meta-web-index', '').strip()
-            self._error = \
-                resp.headers.get('x-container-meta-web-error', '').strip()
-            self._listings = \
-                resp.headers.get('x-container-meta-web-listings', '').strip()
-            self._listings_css = \
-                resp.headers.get('x-container-meta-web-listings-css',
-                                 '').strip()
-            if memcache_client:
-                memcache_client.set(memcache_key,
-                                    (self._index, self._error, self._listings,
-                                     self._listings_css),
-                                    time=self.cache_timeout)
+        self._index = self._error = self._listings = self._listings_css = \
+            self._dir_type = None
+        container_info = get_container_info(env, self.app, swift_source='SW')
+        if is_success(container_info['status']):
+            meta = container_info.get('meta', {})
+            self._index = meta.get('web-index', '').strip()
+            self._error = meta.get('web-error', '').strip()
+            self._listings = meta.get('web-listings', '').strip()
+            self._listings_css = meta.get('web-listings-css', '').strip()
+            self._dir_type = meta.get('web-directory-type', '').strip()
 
     def _listing(self, env, start_response, prefix=None):
         """
@@ -378,13 +371,25 @@ class _StaticWebContext(WSGIContext):
         tmp_env['swift.source'] = 'SW'
         resp = self._app_call(tmp_env)
         status_int = self._get_status_int()
-        if is_success(status_int) or is_redirection(status_int):
-            start_response(self._response_status, self._response_headers,
-                           self._response_exc_info)
-            return resp
-        if status_int != HTTP_NOT_FOUND:
-            return self._error_response(resp, env, start_response)
         self._get_container_info(env)
+        if is_success(status_int) or is_redirection(status_int):
+            # Treat directory marker objects as not found
+            if not self._dir_type:
+                self._dir_type = 'application/directory'
+            content_length = self._response_header_value('content-length')
+            content_length = int(content_length) if content_length else 0
+            if self._response_header_value('content-type') == self._dir_type \
+                    and content_length <= 1:
+                status_int = HTTP_NOT_FOUND
+            else:
+                start_response(self._response_status, self._response_headers,
+                               self._response_exc_info)
+                return resp
+        if status_int != HTTP_NOT_FOUND:
+            # Retaining the previous code's behavior of not using custom error
+            # pages for non-404 errors.
+            self._error = None
+            return self._error_response(resp, env, start_response)
         if not self._listings and not self._index:
             return self.app(env, start_response)
         status_int = HTTP_NOT_FOUND
@@ -458,13 +463,6 @@ class StaticWeb(object):
                 split_path(env['PATH_INFO'], 2, 4, True)
         except ValueError:
             return self.app(env, start_response)
-        if env['REQUEST_METHOD'] in ('PUT', 'POST') and container and not obj:
-            memcache_client = cache_from_env(env)
-            if memcache_client:
-                memcache_key = \
-                    '/staticweb/%s/%s/%s' % (version, account, container)
-                memcache_client.delete(memcache_key)
-            return self.app(env, start_response)
         if env['REQUEST_METHOD'] not in ('HEAD', 'GET'):
             return self.app(env, start_response)
         if env.get('REMOTE_USER') and \
