@@ -13,9 +13,9 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-from time import gmtime, strftime, time
+from time import time
 from traceback import format_exc
-from urllib import quote, unquote
+from urllib import unquote
 from uuid import uuid4
 from hashlib import sha1
 import hmac
@@ -29,7 +29,6 @@ from swift.common.swob import HTTPBadRequest, HTTPForbidden, HTTPNotFound, \
 from swift.common.middleware.acl import clean_acl, parse_acl, referrer_allowed
 from swift.common.utils import cache_from_env, get_logger, \
     split_path, config_true_value
-from swift.common.http import HTTP_CLIENT_CLOSED_REQUEST
 
 
 class TempAuth(object):
@@ -142,12 +141,11 @@ class TempAuth(object):
             # Note: Empty reseller_prefix will match all tokens.
             groups = self.get_groups(env, token)
             if groups:
-                env['REMOTE_USER'] = groups
                 user = groups and groups.split(',', 1)[0] or ''
-                # We know the proxy logs the token, so we augment it just a bit
-                # to also log the authenticated user.
-                env['HTTP_X_AUTH_TOKEN'] = \
-                    '%s,%s' % (user, 's3' if s3 else token)
+                trans_id = env.get('swift.trans_id')
+                self.logger.debug('User: %s uses token %s (trans_id %s)' %
+                                  (user, 's3' if s3 else token, trans_id))
+                env['REMOTE_USER'] = groups
                 env['swift.authorize'] = self.authorize
                 env['swift.clean_acl'] = clean_acl
                 if '.reseller_admin' in groups:
