@@ -19,6 +19,20 @@ Implementation of WSGI Request and Response objects.
 This library has a very similar API to Webob.  It wraps WSGI request
 environments and response values into objects that are more friendly to
 interact with.
+
+Why Swob and not just use WebOb?
+By Michael Barton
+
+We used webob for years. The main problem was that the interface
+wasn't stable. For a while, each of our several test suites required
+a slightly different version of webob to run, and none of them worked
+with the then-current version. It was a huge headache, so we just
+scrapped it.
+
+This is kind of a ton of code, but it's also been a huge relief to
+not have to scramble to add a bunch of code branches all over the
+place to keep Swift working every time webob decides some interface
+needs to change.
 """
 
 from collections import defaultdict
@@ -149,7 +163,7 @@ def _header_property(header):
     (Used by both request and response)
     If a value of None is given, the header is deleted.
 
-    :param header: name of the header, e.g. "Content-Length"
+    :param header: name of the header, e.g. "Transfer-Encoding"
     """
     def getter(self):
         return self.headers.get(header, None)
@@ -231,7 +245,7 @@ class HeaderEnvironProxy(UserDict.DictMixin):
 
 class HeaderKeyDict(dict):
     """
-    A dict that lower-cases all keys on the way in, so as to be
+    A dict that title-cases all keys on the way in, so as to be
     case-insensitive.
     """
     def __init__(self, *args, **kwargs):
@@ -242,30 +256,30 @@ class HeaderKeyDict(dict):
     def update(self, other):
         if hasattr(other, 'keys'):
             for key in other.keys():
-                self[key.lower()] = other[key]
+                self[key.title()] = other[key]
         else:
             for key, value in other:
-                self[key.lower()] = value
+                self[key.title()] = value
 
     def __getitem__(self, key):
-        return dict.get(self, key.lower())
+        return dict.get(self, key.title())
 
     def __setitem__(self, key, value):
         if value is None:
-            self.pop(key.lower(), None)
+            self.pop(key.title(), None)
         elif isinstance(value, unicode):
-            return dict.__setitem__(self, key.lower(), value.encode('utf-8'))
+            return dict.__setitem__(self, key.title(), value.encode('utf-8'))
         else:
-            return dict.__setitem__(self, key.lower(), str(value))
+            return dict.__setitem__(self, key.title(), str(value))
 
     def __contains__(self, key):
-        return dict.__contains__(self, key.lower())
+        return dict.__contains__(self, key.title())
 
     def __delitem__(self, key):
-        return dict.__delitem__(self, key.lower())
+        return dict.__delitem__(self, key.title())
 
     def get(self, key, default=None):
-        return dict.get(self, key.lower(), default)
+        return dict.get(self, key.title(), default)
 
 
 def _resp_status_property():
@@ -803,6 +817,9 @@ class Request(object):
         "Provides the full url of the request"
         return self.host_url + self.path_qs
 
+    def as_referer(self):
+        return self.method + ' ' + self.url
+
     def path_info_pop(self):
         """
         Takes one path portion (delineated by slashes) from the
@@ -888,6 +905,46 @@ class Request(object):
             self.environ.get('SCRIPT_NAME', '') + self.environ['PATH_INFO'],
             minsegs, maxsegs, rest_with_last)
 
+    def message_length(self):
+        """
+        Properly determine the message length for this request. It will return
+        an integer if the headers explicitly contain the message length, or
+        None if the headers don't contain a length. The ValueError exception
+        will be raised if the headers are invalid.
+
+        :raises ValueError: if either transfer-encoding or content-length
+            headers have bad values
+        :raises AttributeError: if the last value of the transfer-encoding
+            header is not "chunked"
+        """
+        te = self.headers.get('transfer-encoding')
+        if te:
+            encodings = te.split(',')
+            if len(encodings) > 1:
+                raise AttributeError('Unsupported Transfer-Coding header'
+                                     ' value specified in Transfer-Encoding'
+                                     ' header')
+            # If there are more than one transfer encoding value, the last
+            # one must be chunked, see RFC 2616 Sec. 3.6
+            if encodings[-1].lower() == 'chunked':
+                chunked = True
+            else:
+                raise ValueError('Invalid Transfer-Encoding header value')
+        else:
+            chunked = False
+        if not chunked:
+            # Because we are not using chunked transfer encoding we can pay
+            # attention to the content-length header.
+            fsize = self.headers.get('content-length', None)
+            if fsize is not None:
+                try:
+                    fsize = int(fsize)
+                except ValueError:
+                    raise ValueError('Invalid Content-Length header value')
+        else:
+            fsize = None
+        return fsize
+
 
 def content_range_header_value(start, stop, size):
     return 'bytes %s-%s/%s' % (start, (stop - 1), size)
@@ -925,7 +982,7 @@ class Response(object):
     charset = _resp_charset_property()
     app_iter = _resp_app_iter_property()
 
-    def __init__(self, body=None, status=200, headers={}, app_iter=None,
+    def __init__(self, body=None, status=200, headers=None, app_iter=None,
                  request=None, conditional_response=False, **kw):
         self.headers = HeaderKeyDict(
             [('Content-Type', 'text/html; charset=UTF-8')])
@@ -939,7 +996,8 @@ class Response(object):
             self.environ = request.environ
         else:
             self.environ = {}
-        self.headers.update(headers)
+        if headers:
+            self.headers.update(headers)
         for key, value in kw.iteritems():
             setattr(self, key, value)
 
@@ -1121,6 +1179,7 @@ HTTPUnprocessableEntity = status_map[422]
 HTTPClientDisconnect = status_map[499]
 HTTPServerError = status_map[500]
 HTTPInternalServerError = status_map[500]
+HTTPNotImplemented = status_map[501]
 HTTPBadGateway = status_map[502]
 HTTPServiceUnavailable = status_map[503]
 HTTPInsufficientStorage = status_map[507]
