@@ -37,7 +37,7 @@ from eventlet.queue import Queue
 from eventlet.timeout import Timeout
 
 from swift.common.utils import ContextPool, normalize_timestamp, \
-    config_true_value, public, json, csv_append
+    config_true_value, public, json, csv_append, GreenthreadSafeIterator
 from swift.common.bufferedhttp import http_connect
 from swift.common.constraints import check_metadata, check_object_creation, \
     CONTAINER_LISTING_LIMIT, MAX_FILE_SIZE
@@ -53,7 +53,7 @@ from swift.proxy.controllers.base import Controller, delay_denial, \
 from swift.common.swob import HTTPAccepted, HTTPBadRequest, HTTPNotFound, \
     HTTPPreconditionFailed, HTTPRequestEntityTooLarge, HTTPRequestTimeout, \
     HTTPServerError, HTTPServiceUnavailable, Request, Response, \
-    HTTPClientDisconnect
+    HTTPClientDisconnect, HTTPNotImplemented
 
 
 def segment_listing_iter(listing):
@@ -71,8 +71,9 @@ def copy_headers_into(from_r, to_r):
     :params from_r: a swob Request or Response
     :params to_r: a swob Request or Response
     """
+    pass_headers = ['x-delete-at']
     for k, v in from_r.headers.items():
-        if k.lower().startswith('x-object-meta-'):
+        if k.lower().startswith('x-object-meta-') or k.lower() in pass_headers:
             to_r.headers[k] = v
 
 
@@ -111,10 +112,11 @@ class SegmentedIterable(object):
         self.container = container
         self.listing = segment_listing_iter(listing)
         self.is_slo = is_slo
-        self.segment = 0
+        self.ratelimit_index = 0
         self.segment_dict = None
         self.segment_peek = None
         self.seek = 0
+        self.length = None
         self.segment_iter = None
         # See NOTE: swift_conn at top of file about this.
         self.segment_iter_swift_conn = None
@@ -132,7 +134,7 @@ class SegmentedIterable(object):
                  segment no longer matches SLO manifest specifications.
         """
         try:
-            self.segment += 1
+            self.ratelimit_index += 1
             self.segment_dict = self.segment_peek or self.listing.next()
             self.segment_peek = None
             if self.container is None:
@@ -140,24 +142,30 @@ class SegmentedIterable(object):
                     self.segment_dict['name'].lstrip('/').split('/', 1)
             else:
                 container, obj = self.container, self.segment_dict['name']
-            partition, nodes = self.controller.app.object_ring.get_nodes(
+            partition = self.controller.app.object_ring.get_part(
                 self.controller.account_name, container, obj)
             path = '/%s/%s/%s' % (self.controller.account_name, container, obj)
             req = Request.blank(path)
-            if self.seek:
-                req.range = 'bytes=%s-' % self.seek
+            if self.seek or (self.length and self.length > 0):
+                bytes_available = self.segment_dict['bytes'] - self.seek
+                range_tail = ''
+                if self.length:
+                    if bytes_available >= self.length:
+                        range_tail = self.seek + self.length - 1
+                        self.length = 0
+                    else:
+                        self.length -= bytes_available
+                if self.seek or range_tail:
+                    req.range = 'bytes=%s-%s' % (self.seek, range_tail)
                 self.seek = 0
-            if not self.is_slo and self.segment > \
+            if not self.is_slo and self.ratelimit_index > \
                     self.controller.app.rate_limit_after_segment:
                 sleep(max(self.next_get_time - time.time(), 0))
             self.next_get_time = time.time() + \
                 1.0 / self.controller.app.rate_limit_segments_per_sec
-            nodes = self.controller.app.sort_nodes(nodes)
             resp = self.controller.GETorHEAD_base(
-                req, _('Object'), partition,
-                self.controller.iter_nodes(partition, nodes,
-                                           self.controller.app.object_ring),
-                path, len(nodes))
+                req, _('Object'), self.controller.app.object_ring, partition,
+                path)
             if self.is_slo and resp.status_int == HTTP_NOT_FOUND:
                 raise SloSegmentError(_(
                     'Could not load object segment %(path)s:'
@@ -173,6 +181,9 @@ class SegmentedIterable(object):
                         '%(path)s etag: %(r_etag)s != %(s_etag)s.' %
                         {'path': path, 'r_etag': resp.etag,
                          's_etag': self.segment_dict['hash']}))
+                if 'X-Static-Large-Object' in resp.headers:
+                    raise SloSegmentError(_(
+                        'SLO can not be made of other SLOs: %s' % path))
             self.segment_iter = resp.app_iter
             # See NOTE: swift_conn at top of file about this.
             self.segment_iter_swift_conn = getattr(resp, 'swift_conn', None)
@@ -188,7 +199,7 @@ class SegmentedIterable(object):
                      'obj': self.controller.object_name, 'err': err})
                 err.swift_logged = True
                 self.response.status_int = HTTP_CONFLICT
-            raise StopIteration('Invalid manifiest segment')
+            raise
         except (Exception, Timeout), err:
             if not getattr(err, 'swift_logged', False):
                 self.controller.app.logger.exception(_(
@@ -216,7 +227,10 @@ class SegmentedIterable(object):
                             chunk = self.segment_iter.next()
                             break
                         except StopIteration:
-                            self._load_next_segment()
+                            if self.length is None or self.length > 0:
+                                self._load_next_segment()
+                            else:
+                                return
                 self.position += len(chunk)
                 yield chunk
         except StopIteration:
@@ -246,7 +260,6 @@ class SegmentedIterable(object):
             if start:
                 self.segment_peek = self.listing.next()
                 while start >= self.position + self.segment_peek['bytes']:
-                    self.segment += 1
                     self.position += self.segment_peek['bytes']
                     self.segment_peek = self.listing.next()
                 self.seek = start - self.position
@@ -254,14 +267,18 @@ class SegmentedIterable(object):
                 start = 0
             if stop is not None:
                 length = stop - start
+                self.length = length
             else:
                 length = None
             for chunk in self:
                 if length is not None:
                     length -= len(chunk)
-                    if length < 0:
-                        # Chop off the extra:
-                        yield chunk[:length]
+                    if length <= 0:
+                        if length < 0:
+                            # Chop off the extra:
+                            yield chunk[:length]
+                        else:
+                            yield chunk
                         break
                 yield chunk
             # See NOTE: swift_conn at top of file about this.
@@ -310,7 +327,7 @@ class ObjectController(Controller):
                 yield item
 
     def _listing_pages_iter(self, lcontainer, lprefix, env):
-        lpartition, lnodes = self.app.container_ring.get_nodes(
+        lpartition = self.app.container_ring.get_part(
             self.account_name, lcontainer)
         marker = ''
         while True:
@@ -322,10 +339,9 @@ class ObjectController(Controller):
             lreq.environ['QUERY_STRING'] = \
                 'format=json&prefix=%s&marker=%s' % (quote(lprefix),
                                                      quote(marker))
-            nodes = self.app.sort_nodes(lnodes)
             lresp = self.GETorHEAD_base(
-                lreq, _('Container'), lpartition, lnodes, lreq.path_info,
-                len(lnodes))
+                lreq, _('Container'), self.app.container_ring, lpartition,
+                lreq.path_info)
             if 'swift.authorize' in env:
                 lreq.acl = lresp.headers.get('x-container-read')
                 aresp = env['swift.authorize'](lreq)
@@ -365,6 +381,44 @@ class ObjectController(Controller):
         except ListingIterNotAuthorized:
             pass
 
+    def iter_nodes_local_first(self, ring, partition):
+        """
+        Yields nodes for a ring partition.
+
+        If the 'write_affinity' setting is non-empty, then this will yield N
+        local nodes (as defined by the write_affinity setting) first, then the
+        rest of the nodes as normal. It is a re-ordering of the nodes such
+        that the local ones come first; no node is omitted. The effect is
+        that the request will be serviced by local object servers first, but
+        nonlocal ones will be employed if not enough local ones are available.
+
+        :param ring: ring to get nodes from
+        :param partition: ring partition to yield nodes for
+        """
+
+        primary_nodes = ring.get_part_nodes(partition)
+        num_locals = self.app.write_affinity_node_count(ring)
+        is_local = self.app.write_affinity_is_local_fn
+
+        if is_local is None:
+            return self.iter_nodes(ring, partition)
+
+        all_nodes = itertools.chain(primary_nodes,
+                                    ring.get_more_nodes(partition))
+        first_n_local_nodes = list(itertools.islice(
+            itertools.ifilter(is_local, all_nodes), num_locals))
+
+        # refresh it; it moved when we computed first_n_local_nodes
+        all_nodes = itertools.chain(primary_nodes,
+                                    ring.get_more_nodes(partition))
+        local_first_node_iter = itertools.chain(
+            first_n_local_nodes,
+            itertools.ifilter(lambda node: node not in first_n_local_nodes,
+                              all_nodes))
+
+        return self.iter_nodes(
+            ring, partition, node_iter=local_first_node_iter)
+
     def is_good_source(self, src):
         """
         Indicates whether or not the request made to the backend found
@@ -378,21 +432,18 @@ class ObjectController(Controller):
 
     def GETorHEAD(self, req):
         """Handle HTTP GET or HEAD requests."""
-        container_info = self.container_info(self.account_name,
-                                             self.container_name)
+        container_info = self.container_info(
+            self.account_name, self.container_name, req)
         req.acl = container_info['read_acl']
         if 'swift.authorize' in req.environ:
             aresp = req.environ['swift.authorize'](req)
             if aresp:
                 return aresp
 
-        partition, nodes = self.app.object_ring.get_nodes(
+        partition = self.app.object_ring.get_part(
             self.account_name, self.container_name, self.object_name)
-        nodes = self.app.sort_nodes(nodes)
         resp = self.GETorHEAD_base(
-            req, _('Object'), partition,
-            self.iter_nodes(partition, nodes, self.app.object_ring),
-            req.path_info, len(nodes))
+            req, _('Object'), self.app.object_ring, partition, req.path_info)
 
         if ';' in resp.headers.get('content-type', ''):
             # strip off swift_bytes from content-type
@@ -402,6 +453,12 @@ class ObjectController(Controller):
                 resp.content_type = content_type
 
         large_object = None
+        if config_true_value(resp.headers.get('x-static-large-object')) and \
+                req.params.get('multipart-manifest') == 'get' and \
+                'X-Copy-From' not in req.headers and \
+                self.app.allow_static_large_object:
+            resp.content_type = 'application/json'
+
         if config_true_value(resp.headers.get('x-static-large-object')) and \
                 req.params.get('multipart-manifest') != 'get' and \
                 self.app.allow_static_large_object:
@@ -420,11 +477,9 @@ class ObjectController(Controller):
                 new_req = req.copy_get()
                 new_req.method = 'GET'
                 new_req.range = None
-                nodes = self.app.sort_nodes(nodes)
                 new_resp = self.GETorHEAD_base(
-                    new_req, _('Object'), partition,
-                    self.iter_nodes(partition, nodes, self.app.object_ring),
-                    req.path_info, len(nodes))
+                    new_req, _('Object'), self.app.object_ring, partition,
+                    req.path_info)
                 if new_resp.status_int // 100 == 2:
                     try:
                         listing = json.loads(new_resp.body)
@@ -568,8 +623,7 @@ class ObjectController(Controller):
             if error_response:
                 return error_response
             container_info = self.container_info(
-                self.account_name, self.container_name,
-                account_autocreate=self.app.account_autocreate)
+                self.account_name, self.container_name, req)
             container_partition = container_info['partition']
             containers = container_info['nodes']
             req.acl = container_info['write_acl']
@@ -590,6 +644,8 @@ class ObjectController(Controller):
                     return HTTPBadRequest(request=req,
                                           content_type='text/plain',
                                           body='Non-integer X-Delete-At')
+                req.environ.setdefault('swift.log_info', []).append(
+                    'x-delete-at:%d' % x_delete_at)
                 delete_at_container = str(
                     x_delete_at /
                     self.app.expiring_objects_container_divisor *
@@ -598,14 +654,14 @@ class ObjectController(Controller):
                     self.app.container_ring.get_nodes(
                         self.app.expiring_objects_account, delete_at_container)
             else:
-                delete_at_part = delete_at_nodes = None
+                delete_at_container = delete_at_part = delete_at_nodes = None
             partition, nodes = self.app.object_ring.get_nodes(
                 self.account_name, self.container_name, self.object_name)
             req.headers['X-Timestamp'] = normalize_timestamp(time.time())
 
             headers = self._backend_requests(
                 req, len(nodes), container_partition, containers,
-                delete_at_part, delete_at_nodes)
+                delete_at_container, delete_at_part, delete_at_nodes)
 
             resp = self.make_requests(req, self.app.object_ring, partition,
                                       'POST', req.path_info, headers)
@@ -613,8 +669,9 @@ class ObjectController(Controller):
 
     def _backend_requests(self, req, n_outgoing,
                           container_partition, containers,
-                          delete_at_partition=None, delete_at_nodes=None):
-        headers = [dict(req.headers.iteritems())
+                          delete_at_container=None, delete_at_partition=None,
+                          delete_at_nodes=None):
+        headers = [self.generate_request_headers(req, additional=req.headers)
                    for _junk in range(n_outgoing)]
 
         for header in headers:
@@ -634,6 +691,7 @@ class ObjectController(Controller):
         for i, node in enumerate(delete_at_nodes or []):
             i = i % len(headers)
 
+            headers[i]['X-Delete-At-Container'] = delete_at_container
             headers[i]['X-Delete-At-Partition'] = delete_at_partition
             headers[i]['X-Delete-At-Host'] = csv_append(
                 headers[i].get('X-Delete-At-Host'),
@@ -681,7 +739,7 @@ class ObjectController(Controller):
                     conn.node = node
                     return conn
                 elif resp.status == HTTP_INSUFFICIENT_STORAGE:
-                    self.error_limit(node)
+                    self.error_limit(node, _('ERROR Insufficient Storage'))
             except:
                 self.exception_occurred(node, _('Object'),
                                         _('Expect: 100-continue on %s') % path)
@@ -692,8 +750,7 @@ class ObjectController(Controller):
     def PUT(self, req):
         """HTTP PUT request handler."""
         container_info = self.container_info(
-            self.account_name, self.container_name,
-            account_autocreate=self.app.account_autocreate)
+            self.account_name, self.container_name, req)
         container_partition = container_info['partition']
         containers = container_info['nodes']
         req.acl = container_info['write_acl']
@@ -705,6 +762,16 @@ class ObjectController(Controller):
                 return aresp
         if not containers:
             return HTTPNotFound(request=req)
+        try:
+            ml = req.message_length()
+        except ValueError as e:
+            return HTTPBadRequest(request=req, content_type='text/plain',
+                                  body=str(e))
+        except AttributeError as e:
+            return HTTPNotImplemented(request=req, content_type='text/plain',
+                                      body=str(e))
+        if ml is not None and ml > MAX_FILE_SIZE:
+            return HTTPRequestEntityTooLarge(request=req)
         if 'x-delete-after' in req.headers:
             try:
                 x_delete_after = int(req.headers['x-delete-after'])
@@ -713,25 +780,6 @@ class ObjectController(Controller):
                                           content_type='text/plain',
                                           body='Non-integer X-Delete-After')
             req.headers['x-delete-at'] = '%d' % (time.time() + x_delete_after)
-        if 'x-delete-at' in req.headers:
-            try:
-                x_delete_at = int(req.headers['x-delete-at'])
-                if x_delete_at < time.time():
-                    return HTTPBadRequest(
-                        body='X-Delete-At in past', request=req,
-                        content_type='text/plain')
-            except ValueError:
-                return HTTPBadRequest(request=req, content_type='text/plain',
-                                      body='Non-integer X-Delete-At')
-            delete_at_container = str(
-                x_delete_at /
-                self.app.expiring_objects_container_divisor *
-                self.app.expiring_objects_container_divisor)
-            delete_at_part, delete_at_nodes = \
-                self.app.container_ring.get_nodes(
-                    self.app.expiring_objects_account, delete_at_container)
-        else:
-            delete_at_part = delete_at_nodes = None
         partition, nodes = self.app.object_ring.get_nodes(
             self.account_name, self.container_name, self.object_name)
         # do a HEAD request for container sync and checking object versions
@@ -740,8 +788,9 @@ class ObjectController(Controller):
                  req.environ.get('swift_versioned_copy')):
             hreq = Request.blank(req.path_info, headers={'X-Newest': 'True'},
                                  environ={'REQUEST_METHOD': 'HEAD'})
-            hresp = self.GETorHEAD_base(hreq, _('Object'), partition, nodes,
-                                        hreq.path_info, len(nodes))
+            hresp = self.GETorHEAD_base(
+                hreq, _('Object'), self.app.object_ring, partition,
+                hreq.path_info)
         # Used by container sync feature
         if 'x-timestamp' in req.headers:
             try:
@@ -863,13 +912,38 @@ class ObjectController(Controller):
                     source_resp.headers['X-Static-Large-Object']
 
             req = new_req
-        node_iter = self.iter_nodes(partition, nodes, self.app.object_ring)
+
+        if 'x-delete-at' in req.headers:
+            try:
+                x_delete_at = int(req.headers['x-delete-at'])
+                if x_delete_at < time.time():
+                    return HTTPBadRequest(
+                        body='X-Delete-At in past', request=req,
+                        content_type='text/plain')
+            except ValueError:
+                return HTTPBadRequest(request=req, content_type='text/plain',
+                                      body='Non-integer X-Delete-At')
+            req.environ.setdefault('swift.log_info', []).append(
+                'x-delete-at:%d' % x_delete_at)
+            delete_at_container = str(
+                x_delete_at /
+                self.app.expiring_objects_container_divisor *
+                self.app.expiring_objects_container_divisor)
+            delete_at_part, delete_at_nodes = \
+                self.app.container_ring.get_nodes(
+                    self.app.expiring_objects_account, delete_at_container)
+        else:
+            delete_at_container = delete_at_part = delete_at_nodes = None
+
+        node_iter = GreenthreadSafeIterator(
+            self.iter_nodes_local_first(self.app.object_ring, partition))
         pile = GreenPile(len(nodes))
-        chunked = req.headers.get('transfer-encoding')
+        te = req.headers.get('transfer-encoding', '')
+        chunked = ('chunked' in te)
 
         outgoing_headers = self._backend_requests(
             req, len(nodes), container_partition, containers,
-            delete_at_part, delete_at_nodes)
+            delete_at_container, delete_at_part, delete_at_nodes)
 
         for nheaders in outgoing_headers:
             # RFC2616:8.2.3 disallows 100-continue without a body
@@ -966,7 +1040,7 @@ class ObjectController(Controller):
             self.app.logger.error(
                 _('Object servers returned %s mismatched etags'), len(etags))
             return HTTPServerError(request=req)
-        etag = len(etags) and etags.pop() or None
+        etag = etags.pop() if len(etags) else None
         while len(statuses) < len(nodes):
             statuses.append(HTTP_SERVICE_UNAVAILABLE)
             reasons.append('')
@@ -988,8 +1062,8 @@ class ObjectController(Controller):
     @delay_denial
     def DELETE(self, req):
         """HTTP DELETE request handler."""
-        container_info = self.container_info(self.account_name,
-                                             self.container_name)
+        container_info = self.container_info(
+            self.account_name, self.container_name, req)
         container_partition = container_info['partition']
         containers = container_info['nodes']
         req.acl = container_info['write_acl']
@@ -1040,8 +1114,8 @@ class ObjectController(Controller):
                 self.container_name = lcontainer
                 self.object_name = last_item['name']
                 new_del_req = Request.blank(copy_path, environ=req.environ)
-                container_info = self.container_info(self.account_name,
-                                                     self.container_name)
+                container_info = self.container_info(
+                    self.account_name, self.container_name, req)
                 container_partition = container_info['partition']
                 containers = container_info['nodes']
                 new_del_req.acl = container_info['write_acl']
