@@ -111,6 +111,13 @@ class TestUntar(unittest.TestCase):
         self.app.calls = 0
         rmtree(self.testdir)
 
+    def handle_extract_and_iter(self, req, compress_format,
+                                out_content_type='application/json'):
+        resp_body = ''.join(
+            self.bulk.handle_extract_iter(req, compress_format,
+                                          out_content_type=out_content_type))
+        return resp_body
+
     def test_create_container_for_path(self):
         req = Request.blank('/')
         self.assertEquals(
@@ -142,33 +149,41 @@ class TestUntar(unittest.TestCase):
                                mode=mode)
             tar.add(os.path.join(self.testdir, base_name))
             tar.close()
-            req = Request.blank('/tar_works/acc/cont/',
-                                headers={'Accept': 'application/json'})
+            req = Request.blank('/tar_works/acc/cont/')
             req.environ['wsgi.input'] = open(
                 os.path.join(self.testdir, 'tar_works.tar' + extension))
             req.headers['transfer-encoding'] = 'chunked'
-            resp = self.bulk.handle_extract(req, compress_format)
-            resp_data = json.loads(resp.body)
+            resp_body = self.handle_extract_and_iter(req, compress_format)
+            resp_data = json.loads(resp_body)
             self.assertEquals(resp_data['Number Files Created'], 6)
 
             # test out xml
-            req = Request.blank('/tar_works/acc/cont/',
-                                headers={'Accept': 'application/xml'})
+            req = Request.blank('/tar_works/acc/cont/')
             req.environ['wsgi.input'] = open(
                 os.path.join(self.testdir, 'tar_works.tar' + extension))
             req.headers['transfer-encoding'] = 'chunked'
-            resp = self.bulk.handle_extract(req, compress_format)
-            self.assertEquals(resp.status_int, 201)
+            resp_body = self.handle_extract_and_iter(
+                req, compress_format, 'application/xml')
+            self.assert_('<response_status>201 Created</response_status>' in
+                         resp_body)
             self.assert_('<number_files_created>6</number_files_created>' in
-                         resp.body)
+                         resp_body)
 
             # test out nonexistent format
-            req = Request.blank('/tar_works/acc/cont/',
+            req = Request.blank('/tar_works/acc/cont/?extract-archive=tar',
                                 headers={'Accept': 'good_xml'})
+            req.environ['REQUEST_METHOD'] = 'PUT'
             req.environ['wsgi.input'] = open(
                 os.path.join(self.testdir, 'tar_works.tar' + extension))
-            resp = self.bulk.handle_extract(req, compress_format)
-            self.assertEquals(resp.status_int, 406)
+            req.headers['transfer-encoding'] = 'chunked'
+
+            def fake_start_response(*args, **kwargs):
+                pass
+
+            app_iter = self.bulk(req.environ, fake_start_response)
+            resp_body = ''.join([i for i in app_iter])
+
+            self.assert_('Response Status: 406' in resp_body)
 
     def test_extract_call(self):
         base_name = 'base_works_gz'
@@ -198,7 +213,8 @@ class TestUntar(unittest.TestCase):
             os.path.join(self.testdir, 'tar_works.tar.gz'))
         req.headers['transfer-encoding'] = 'Chunked'
         req.method = 'PUT'
-        self.bulk(req.environ, fake_start_response)
+        app_iter = self.bulk(req.environ, fake_start_response)
+        list(app_iter)  # iter over resp
         self.assertEquals(self.app.calls, 7)
 
         self.app.calls = 0
@@ -221,18 +237,19 @@ class TestUntar(unittest.TestCase):
         req.headers['transfer-encoding'] = 'Chunked'
         req.environ['wsgi.input'] = open(
             os.path.join(self.testdir, 'tar_works.tar'))
-        t = self.bulk(req.environ, fake_start_response)
+        app_iter = self.bulk(req.environ, fake_start_response)
+        list(app_iter)  # iter over resp
         self.assertEquals(self.app.calls, 7)
 
     def test_bad_container(self):
         req = Request.blank('/invalid/', body='')
-        resp = self.bulk.handle_extract(req, '')
-        self.assertEquals(resp.status_int, 404)
+        resp_body = self.handle_extract_and_iter(req, '')
+        self.assertTrue('404 Not Found' in resp_body)
 
     def test_content_length_required(self):
         req = Request.blank('/create_cont_fail/acc/cont')
-        resp = self.bulk.handle_extract(req, '')
-        self.assertEquals(resp.status_int, 411)
+        resp_body = self.handle_extract_and_iter(req, '')
+        self.assertTrue('411 Length Required' in resp_body)
 
     def build_tar(self, dir_tree=None):
         if not dir_tree:
@@ -254,59 +271,76 @@ class TestUntar(unittest.TestCase):
             {'base_fails1': [{'sub_dir1': ['sub1_file1']},
                              {'sub_dir2': ['sub2_file1', 'sub2_file2']},
                              {'sub_dir3': [{'sub4_dir1': 'sub4_file1'}]}]}]
-        tar = self.build_tar(dir_tree)
-        req = Request.blank('/tar_works/acc/',
+        self.build_tar(dir_tree)
+        req = Request.blank('/tar_works/acc/')
+        req.environ['wsgi.input'] = open(os.path.join(self.testdir,
+                                                      'tar_fails.tar'))
+        req.headers['transfer-encoding'] = 'chunked'
+        resp_body = self.handle_extract_and_iter(req, '')
+        resp_data = json.loads(resp_body)
+        self.assertEquals(resp_data['Number Files Created'], 4)
+
+    def test_extract_tar_fail_cont_401(self):
+        self.build_tar()
+        req = Request.blank('/unauth/acc/',
                             headers={'Accept': 'application/json'})
         req.environ['wsgi.input'] = open(os.path.join(self.testdir,
                                                       'tar_fails.tar'))
         req.headers['transfer-encoding'] = 'chunked'
-        resp = self.bulk.handle_extract(req, '')
-        resp_data = json.loads(resp.body)
-        self.assertEquals(resp_data['Number Files Created'], 4)
-
-    def test_extract_tar_fail_cont_401(self):
-        tar = self.build_tar()
-        req = Request.blank('/unauth/acc/')
-        req.environ['wsgi.input'] = open(os.path.join(self.testdir,
-                                                      'tar_fails.tar'))
-        req.headers['transfer-encoding'] = 'chunked'
-        resp = self.bulk.handle_extract(req, '')
-        self.assertEquals(resp.status_int, 401)
+        resp_body = self.handle_extract_and_iter(req, '')
+        self.assertEquals(self.app.calls, 1)
+        resp_data = json.loads(resp_body)
+        self.assertEquals(resp_data['Response Status'], '401 Unauthorized')
+        self.assertEquals(
+            resp_data['Errors'],
+            [['base_fails1/sub_dir1/sub1_file1', '401 Unauthorized']])
 
     def test_extract_tar_fail_obj_401(self):
-        tar = self.build_tar()
-        req = Request.blank('/create_obj_unauth/acc/cont/')
+        self.build_tar()
+        req = Request.blank('/create_obj_unauth/acc/cont/',
+                            headers={'Accept': 'application/json'})
         req.environ['wsgi.input'] = open(os.path.join(self.testdir,
                                                       'tar_fails.tar'))
         req.headers['transfer-encoding'] = 'chunked'
-        resp = self.bulk.handle_extract(req, '')
-        self.assertEquals(resp.status_int, 401)
+        resp_body = self.handle_extract_and_iter(req, '')
+        self.assertEquals(self.app.calls, 2)
+        resp_data = json.loads(resp_body)
+        self.assertEquals(resp_data['Response Status'], '401 Unauthorized')
+        self.assertEquals(
+            resp_data['Errors'],
+            [['cont/base_fails1/sub_dir1/sub1_file1', '401 Unauthorized']])
 
     def test_extract_tar_fail_obj_name_len(self):
-        tar = self.build_tar()
+        self.build_tar()
         req = Request.blank('/tar_works/acc/cont/',
                             headers={'Accept': 'application/json'})
         req.environ['wsgi.input'] = open(os.path.join(self.testdir,
                                                       'tar_fails.tar'))
         req.headers['transfer-encoding'] = 'chunked'
-        resp = self.bulk.handle_extract(req, '')
-        resp_data = json.loads(resp.body)
+        resp_body = self.handle_extract_and_iter(req, '')
+        self.assertEquals(self.app.calls, 6)
+        resp_data = json.loads(resp_body)
         self.assertEquals(resp_data['Number Files Created'], 4)
-        self.assertEquals(resp_data['Errors'][0][0],
-                          '/tar_works/acc/cont/base_fails1/' + ('f' * 101))
+        self.assertEquals(
+            resp_data['Errors'],
+            [['cont/base_fails1/' + ('f' * 101), '400 Bad Request']])
 
     def test_extract_tar_fail_compress_type(self):
-        tar = self.build_tar()
-        req = Request.blank('/tar_works/acc/cont/')
+        self.build_tar()
+        req = Request.blank('/tar_works/acc/cont/',
+                            headers={'Accept': 'application/json'})
         req.environ['wsgi.input'] = open(os.path.join(self.testdir,
                                                       'tar_fails.tar'))
         req.headers['transfer-encoding'] = 'chunked'
-        resp = self.bulk.handle_extract(req, 'gz')
-        self.assertEquals(resp.status_int, 400)
+        resp_body = self.handle_extract_and_iter(req, 'gz')
         self.assertEquals(self.app.calls, 0)
+        resp_data = json.loads(resp_body)
+        self.assertEquals(resp_data['Response Status'], '400 Bad Request')
+        self.assertEquals(
+            resp_data['Response Body'], 'Invalid Tar File: not a gzip file')
 
-    def test_extract_tar_fail_max_file_name_length(self):
-        tar = self.build_tar()
+    def test_extract_tar_fail_max_failed_extractions(self):
+        self.build_tar()
         with patch.object(self.bulk, 'max_failed_extractions', 1):
             self.app.calls = 0
             req = Request.blank('/tar_works/acc/cont/',
@@ -314,11 +348,13 @@ class TestUntar(unittest.TestCase):
             req.environ['wsgi.input'] = open(os.path.join(self.testdir,
                                                           'tar_fails.tar'))
             req.headers['transfer-encoding'] = 'chunked'
-            resp = self.bulk.handle_extract(req, '')
-            resp_data = json.loads(resp.body)
+            resp_body = self.handle_extract_and_iter(req, '')
             self.assertEquals(self.app.calls, 5)
-            self.assertEquals(resp_data['Errors'][0][0],
-                              '/tar_works/acc/cont/base_fails1/' + ('f' * 101))
+            resp_data = json.loads(resp_body)
+            self.assertEquals(resp_data['Number Files Created'], 3)
+            self.assertEquals(
+                resp_data['Errors'],
+                [['cont/base_fails1/' + ('f' * 101), '400 Bad Request']])
 
     @patch.object(bulk, 'MAX_FILE_SIZE', 4)
     def test_extract_tar_fail_max_file_size(self):
@@ -336,44 +372,51 @@ class TestUntar(unittest.TestCase):
         req.environ['wsgi.input'] = open(
             os.path.join(self.testdir, 'tar_works.tar'))
         req.headers['transfer-encoding'] = 'chunked'
-        resp = self.bulk.handle_extract(req, '')
-        resp_data = json.loads(resp.body)
-        self.assert_(resp_data['Errors'][0][1].startswith('413'))
+        resp_body = self.handle_extract_and_iter(req, '')
+        resp_data = json.loads(resp_body)
+        self.assertEquals(
+            resp_data['Errors'],
+            [['cont' + self.testdir + '/test/sub_dir1/sub1_file1',
+              '413 Request Entity Too Large']])
 
     def test_extract_tar_fail_max_cont(self):
         dir_tree = [{'sub_dir1': ['sub1_file1']},
                     {'sub_dir2': ['sub2_file1', 'sub2_file2']},
                     'f' * 101,
                     {'sub_dir3': [{'sub4_dir1': 'sub4_file1'}]}]
-        tar = self.build_tar(dir_tree)
+        self.build_tar(dir_tree)
         with patch.object(self.bulk, 'max_containers', 1):
             self.app.calls = 0
             body = open(os.path.join(self.testdir, 'tar_fails.tar')).read()
-            req = Request.blank('/tar_works/acc/', body=body)
+            req = Request.blank('/tar_works/acc/', body=body,
+                                headers={'Accept': 'application/json'})
             req.headers['transfer-encoding'] = 'chunked'
-            resp = self.bulk.handle_extract(req, '')
+            resp_body = self.handle_extract_and_iter(req, '')
             self.assertEquals(self.app.calls, 3)
-            self.assertEquals(resp.status_int, 400)
+            resp_data = json.loads(resp_body)
+            self.assertEquals(resp_data['Response Status'], '400 Bad Request')
+            self.assertEquals(
+                resp_data['Response Body'],
+                'More than 1 base level containers in tar.')
 
     def test_extract_tar_fail_create_cont(self):
         dir_tree = [{'base_fails1': [
             {'sub_dir1': ['sub1_file1']},
             {'sub_dir2': ['sub2_file1', 'sub2_file2']},
-            'f\xde',
             {'./sub_dir3': [{'sub4_dir1': 'sub4_file1'}]}]}]
-        tar = self.build_tar(dir_tree)
+        self.build_tar(dir_tree)
         req = Request.blank('/create_cont_fail/acc/cont/',
                             headers={'Accept': 'application/json'})
         req.environ['wsgi.input'] = open(os.path.join(self.testdir,
                                                       'tar_fails.tar'))
         req.headers['transfer-encoding'] = 'chunked'
-        resp = self.bulk.handle_extract(req, '')
-        resp_data = json.loads(resp.body)
+        resp_body = self.handle_extract_and_iter(req, '')
+        resp_data = json.loads(resp_body)
         self.assertEquals(self.app.calls, 4)
-        self.assertEquals(len(resp_data['Errors']), 5)
+        self.assertEquals(len(resp_data['Errors']), 4)
 
     def test_extract_tar_fail_create_cont_value_err(self):
-        tar = self.build_tar()
+        self.build_tar()
         req = Request.blank('/create_cont_fail/acc/cont/',
                             headers={'Accept': 'application/json'})
         req.environ['wsgi.input'] = open(os.path.join(self.testdir,
@@ -384,14 +427,38 @@ class TestUntar(unittest.TestCase):
             raise ValueError('Test')
 
         with patch.object(self.bulk, 'create_container', bad_create):
-            resp = self.bulk.handle_extract(req, '')
-            resp_data = json.loads(resp.body)
+            resp_body = self.handle_extract_and_iter(req, '')
+            resp_data = json.loads(resp_body)
             self.assertEquals(self.app.calls, 0)
             self.assertEquals(len(resp_data['Errors']), 5)
+            self.assertEquals(
+                resp_data['Errors'][0],
+                ['cont/base_fails1/sub_dir1/sub1_file1', '400 Bad Request'])
+
+    def test_extract_tar_fail_unicode(self):
+        dir_tree = [{'sub_dir1': ['sub1_file1']},
+                    {'sub_dir2': ['sub2\xdefile1', 'sub2_file2']},
+                    {'sub_\xdedir3': [{'sub4_dir1': 'sub4_file1'}]}]
+        self.build_tar(dir_tree)
+        req = Request.blank('/tar_works/acc/',
+                            headers={'Accept': 'application/json'})
+        req.environ['wsgi.input'] = open(os.path.join(self.testdir,
+                                                      'tar_fails.tar'))
+        req.headers['transfer-encoding'] = 'chunked'
+        resp_body = self.handle_extract_and_iter(req, '')
+        resp_data = json.loads(resp_body)
+        self.assertEquals(self.app.calls, 4)
+        self.assertEquals(resp_data['Number Files Created'], 2)
+        self.assertEquals(resp_data['Response Status'], '400 Bad Request')
+        self.assertEquals(
+            resp_data['Errors'],
+            [['sub_dir2/sub2%DEfile1', '412 Precondition Failed'],
+             ['sub_%DEdir3/sub4_dir1/sub4_file1', '412 Precondition Failed']])
 
     def test_get_response_body(self):
-        self.assertRaises(
-            HTTPException, bulk.get_response_body, 'badformat', {}, [])
+        txt_body = bulk.get_response_body(
+            'bad_formay', {'hey': 'there'}, [['json > xml', '202 Accepted']])
+        self.assert_('hey: there' in txt_body)
         xml_body = bulk.get_response_body(
             'text/xml', {'hey': 'there'}, [['json > xml', '202 Accepted']])
         self.assert_('&gt' in xml_body)
@@ -407,44 +474,47 @@ class TestDelete(unittest.TestCase):
         self.app.calls = 0
         self.app.delete_paths = []
 
+    def handle_delete_and_iter(self, req, out_content_type='application/json'):
+        resp_body = ''.join(self.bulk.handle_delete_iter(
+            req, out_content_type=out_content_type))
+        return resp_body
+
     def test_bulk_delete_works(self):
         req = Request.blank('/delete_works/AUTH_Acc', body='/c/f\n/c/f404',
                             headers={'Accept': 'application/json'})
         req.method = 'DELETE'
-        resp = self.bulk.handle_delete(req)
+        resp_body = self.handle_delete_and_iter(req)
         self.assertEquals(
             self.app.delete_paths,
             ['/delete_works/AUTH_Acc/c/f', '/delete_works/AUTH_Acc/c/f404'])
         self.assertEquals(self.app.calls, 2)
-        resp_data = json.loads(resp.body)
+        resp_data = json.loads(resp_body)
         self.assertEquals(resp_data['Number Deleted'], 1)
         self.assertEquals(resp_data['Number Not Found'], 1)
 
-    def test_bulk_delete_bad_accept_and_content_type(self):
+    def test_bulk_delete_bad_content_type(self):
         req = Request.blank('/delete_works/AUTH_Acc',
                             headers={'Accept': 'badformat'})
 
-        req.method = 'DELETE'
-        req.environ['wsgi.input'] = StringIO('/c/f\n/c/f404')
-        resp = self.bulk.handle_delete(req)
-        self.assertEquals(resp.status_int, 406)
-
         req = Request.blank('/delete_works/AUTH_Acc',
                             headers={'Accept': 'application/json',
                                      'Content-Type': 'text/xml'})
         req.method = 'DELETE'
         req.environ['wsgi.input'] = StringIO('/c/f\n/c/f404')
-        resp = self.bulk.handle_delete(req)
-        self.assertEquals(resp.status_int, 406)
+        resp_body = self.handle_delete_and_iter(req)
+        resp_data = json.loads(resp_body)
+        self.assertEquals(resp_data['Response Status'], '406 Not Acceptable')
 
-    def test_bulk_delete_call(self):
+    def test_bulk_delete_call_and_content_type(self):
         def fake_start_response(*args, **kwargs):
-            pass
+            self.assertEquals(args[1][0], ('Content-Type', 'application/json'))
+
         req = Request.blank('/delete_works/AUTH_Acc?bulk-delete')
         req.method = 'DELETE'
         req.headers['Transfer-Encoding'] = 'chunked'
+        req.headers['Accept'] = 'application/json'
         req.environ['wsgi.input'] = StringIO('/c/f')
-        self.bulk(req.environ, fake_start_response)
+        list(self.bulk(req.environ, fake_start_response))  # iterate over resp
         self.assertEquals(
             self.app.delete_paths, ['/delete_works/AUTH_Acc/c/f'])
         self.assertEquals(self.app.calls, 1)
@@ -474,14 +544,14 @@ class TestDelete(unittest.TestCase):
                             body='/c/f\n\n\n/c/f404\n\n\n/c/%2525',
                             headers={'Accept': 'application/json'})
         req.method = 'DELETE'
-        resp = self.bulk.handle_delete(req)
+        resp_body = self.handle_delete_and_iter(req)
         self.assertEquals(
             self.app.delete_paths,
             ['/delete_works/AUTH_Acc/c/f',
              '/delete_works/AUTH_Acc/c/f404',
              '/delete_works/AUTH_Acc/c/%25'])
         self.assertEquals(self.app.calls, 3)
-        resp_data = json.loads(resp.body)
+        resp_data = json.loads(resp_body)
         self.assertEquals(resp_data['Number Deleted'], 2)
         self.assertEquals(resp_data['Number Not Found'], 1)
 
@@ -491,23 +561,8 @@ class TestDelete(unittest.TestCase):
         data = '\n\n' * self.bulk.max_deletes_per_request
         req.environ['wsgi.input'] = StringIO(data)
         req.content_length = len(data)
-        try:
-            self.bulk.handle_delete(req)
-        except HTTPException, err:
-            self.assertEquals(err.status_int, 413)
-        else:
-            self.fail('413 not raised')
-
-    def test_bulk_delete_raised_error(self):
-        def fake_start_response(*args, **kwargs):
-            self.assertTrue(args[0].startswith('413'))
-
-        req = Request.blank('/delete_works/AUTH_Acc?bulk-delete')
-        req.method = 'DELETE'
-        data = '\n\n' * self.bulk.max_deletes_per_request
-        req.environ['wsgi.input'] = StringIO(data)
-        req.content_length = len(data)
-        self.bulk(req.environ, fake_start_response)
+        resp_body = self.handle_delete_and_iter(req)
+        self.assertTrue('413 Request Entity Too Large' in resp_body)
 
     def test_bulk_delete_works_unicode(self):
         body = (u'/c/ obj \u2661\r\n'.encode('utf8') +
@@ -516,80 +571,87 @@ class TestDelete(unittest.TestCase):
         req = Request.blank('/delete_works/AUTH_Acc', body=body,
                             headers={'Accept': 'application/json'})
         req.method = 'DELETE'
-        resp = self.bulk.handle_delete(req)
+        resp_body = self.handle_delete_and_iter(req)
         self.assertEquals(
             self.app.delete_paths,
             ['/delete_works/AUTH_Acc/c/ obj \xe2\x99\xa1',
              '/delete_works/AUTH_Acc/c/ objbadutf8'])
 
         self.assertEquals(self.app.calls, 2)
-        resp_data = json.loads(resp.body)
+        resp_data = json.loads(resp_body)
         self.assertEquals(resp_data['Number Deleted'], 1)
         self.assertEquals(len(resp_data['Errors']), 2)
         self.assertEquals(
             resp_data['Errors'],
-            [[urllib.quote('/delete_works/AUTH_Acc/c/ objbadutf8'),
-              '412 Precondition Failed'],
-             [urllib.quote('/delete_works/AUTH_Acc/c/f\xdebadutf8'),
-              '412 Precondition Failed']])
+            [[urllib.quote('c/ objbadutf8'), '412 Precondition Failed'],
+             [urllib.quote('/c/f\xdebadutf8'), '412 Precondition Failed']])
 
     def test_bulk_delete_no_body(self):
         req = Request.blank('/unauth/AUTH_acc/')
-        self.assertRaises(HTTPException, self.bulk.handle_delete, req)
+        resp_body = self.handle_delete_and_iter(req)
+        self.assertTrue('411 Length Required' in resp_body)
 
     def test_bulk_delete_no_files_in_body(self):
         req = Request.blank('/unauth/AUTH_acc/', body=' ')
-        resp = self.bulk.handle_delete(req)
-        self.assertEquals(resp.status_int, 400)
+        resp_body = self.handle_delete_and_iter(req)
+        self.assertTrue('400 Bad Request' in resp_body)
 
     def test_bulk_delete_unauth(self):
-        req = Request.blank('/unauth/AUTH_acc/', body='/c/f\n')
+        req = Request.blank('/unauth/AUTH_acc/', body='/c/f\n/c/f2\n',
+                            headers={'Accept': 'application/json'})
         req.method = 'DELETE'
-        resp = self.bulk.handle_delete(req)
-        self.assertEquals(resp.status_int, 401)
+        resp_body = self.handle_delete_and_iter(req)
+        self.assertEquals(self.app.calls, 1)
+        resp_data = json.loads(resp_body)
+        self.assertEquals(resp_data['Errors'], [['/c/f', '401 Unauthorized']])
+        self.assertEquals(resp_data['Response Status'], '401 Unauthorized')
 
     def test_bulk_delete_500_resp(self):
-        req = Request.blank('/broke/AUTH_acc/', body='/c/f\n')
+        req = Request.blank('/broke/AUTH_acc/', body='/c/f\nc/f2\n',
+                            headers={'Accept': 'application/json'})
         req.method = 'DELETE'
-        resp = self.bulk.handle_delete(req)
-        self.assertEquals(resp.status_int, 502)
+        resp_body = self.handle_delete_and_iter(req)
+        resp_data = json.loads(resp_body)
+        self.assertEquals(
+            resp_data['Errors'],
+            [['/c/f', '500 Internal Error'], ['c/f2', '500 Internal Error']])
+        self.assertEquals(resp_data['Response Status'], '502 Bad Gateway')
 
     def test_bulk_delete_bad_path(self):
         req = Request.blank('/delete_cont_fail/')
-        resp = self.bulk.handle_delete(req)
-        self.assertEquals(resp.status_int, 404)
+        resp_body = self.handle_delete_and_iter(req)
+        self.assertTrue('404 Not Found' in resp_body)
 
     def test_bulk_delete_container_delete(self):
         req = Request.blank('/delete_cont_fail/AUTH_Acc', body='c\n',
                             headers={'Accept': 'application/json'})
         req.method = 'DELETE'
-        resp = self.bulk.handle_delete(req)
-        resp_data = json.loads(resp.body)
+        resp_body = self.handle_delete_and_iter(req)
+        resp_data = json.loads(resp_body)
         self.assertEquals(resp_data['Number Deleted'], 0)
-        self.assertEquals(resp_data['Errors'][0][1], '409 Conflict')
+        self.assertEquals(resp_data['Errors'], [['c', '409 Conflict']])
+        self.assertEquals(resp_data['Response Status'], '400 Bad Request')
 
     def test_bulk_delete_bad_file_too_long(self):
         req = Request.blank('/delete_works/AUTH_Acc',
                             headers={'Accept': 'application/json'})
         req.method = 'DELETE'
-        data = '/c/f\nc/' + ('1' * bulk.MAX_PATH_LENGTH) + '\n/c/f'
+        bad_file = 'c/' + ('1' * bulk.MAX_PATH_LENGTH)
+        data = '/c/f\n' + bad_file + '\n/c/f'
         req.environ['wsgi.input'] = StringIO(data)
         req.headers['Transfer-Encoding'] = 'chunked'
-        resp = self.bulk.handle_delete(req)
-        resp_data = json.loads(resp.body)
+        resp_body = self.handle_delete_and_iter(req)
+        resp_data = json.loads(resp_body)
         self.assertEquals(resp_data['Number Deleted'], 2)
-        self.assertEquals(resp_data['Errors'][0][1], '400 Bad Request')
+        self.assertEquals(resp_data['Errors'], [[bad_file, '400 Bad Request']])
+        self.assertEquals(resp_data['Response Status'], '400 Bad Request')
 
     def test_bulk_delete_bad_file_over_twice_max_length(self):
         body = '/c/f\nc/' + ('123456' * bulk.MAX_PATH_LENGTH) + '\n'
         req = Request.blank('/delete_works/AUTH_Acc', body=body)
         req.method = 'DELETE'
-        try:
-            self.bulk.handle_delete(req)
-        except HTTPException, err:
-            self.assertEquals(err.status_int, 400)
-        else:
-            self.fail('400 not raised')
+        resp_body = self.handle_delete_and_iter(req)
+        self.assertTrue('400 Bad Request' in resp_body)
 
 if __name__ == '__main__':
     unittest.main()
