@@ -22,7 +22,6 @@ from contextlib import contextmanager
 
 from swift.common.swob import Request, Response
 from swift.common.middleware import staticweb
-from test.unit import FakeLogger
 
 
 class FakeMemcache(object):
@@ -53,6 +52,47 @@ class FakeMemcache(object):
         return True
 
 
+meta_map = {
+    'c1': {'status': 401},
+    'c2': {},
+    'c3': {'meta': {'web-index': 'index.html',
+                    'web-listings': 't'}},
+    'c3b': {'meta': {'web-index': 'index.html',
+                     'web-listings': 't'}},
+    'c4': {'meta': {'web-index': 'index.html',
+                    'web-error': 'error.html',
+                    'web-listings': 't',
+                    'web-listings-css': 'listing.css',
+                    'web-directory-type': 'text/dir'}},
+    'c5': {'meta': {'web-index': 'index.html',
+                    'web-error': 'error.html',
+                    'web-listings': 't',
+                    'web-listings-css': 'listing.css'}},
+    'c6': {'meta': {'web-listings': 't'}},
+    'c7': {'meta': {'web-listings': 'f'}},
+    'c8': {'meta': {'web-error': 'error.html',
+                    'web-listings': 't',
+                    'web-listings-css':
+                    'http://localhost/stylesheets/listing.css'}},
+    'c9': {'meta': {'web-error': 'error.html',
+                    'web-listings': 't',
+                    'web-listings-css':
+                    '/absolute/listing.css'}},
+    'c10': {'meta': {'web-listings': 't'}},
+    'c11': {'meta': {'web-index': 'index.html'}},
+    'c11a': {'meta': {'web-index': 'index.html',
+             'web-directory-type': 'text/directory'}},
+}
+
+
+def mock_get_container_info(env, app, swift_source='SW'):
+    container = env['PATH_INFO'].rstrip('/').split('/')[3]
+    container_info = meta_map[container]
+    container_info.setdefault('status', 200)
+    container_info.setdefault('read_acl', '.r:*')
+    return container_info
+
+
 class FakeApp(object):
 
     def __init__(self, status_headers_body_iter=None):
@@ -71,15 +111,11 @@ class FakeApp(object):
         elif env['PATH_INFO'] == '/v1/a/c1':
             return Response(status='401 Unauthorized')(env, start_response)
         elif env['PATH_INFO'] == '/v1/a/c2':
-            return self.listing(env, start_response,
-                                {'x-container-read': '.r:*'})
+            return self.listing(env, start_response)
         elif env['PATH_INFO'] == '/v1/a/c2/one.txt':
             return Response(status='404 Not Found')(env, start_response)
         elif env['PATH_INFO'] == '/v1/a/c3':
-            return self.listing(env, start_response,
-                                {'x-container-read': '.r:*',
-                                 'x-container-meta-web-index': 'index.html',
-                                 'x-container-meta-web-listings': 't'})
+            return self.listing(env, start_response)
         elif env['PATH_INFO'] == '/v1/a/c3/index.html':
             return Response(status='200 Ok', body='''
 <html>
@@ -94,10 +130,7 @@ class FakeApp(object):
 </html>
             ''')(env, start_response)
         elif env['PATH_INFO'] == '/v1/a/c3b':
-            return self.listing(env, start_response,
-                                {'x-container-read': '.r:*',
-                                 'x-container-meta-web-index': 'index.html',
-                                 'x-container-meta-web-listings': 't'})
+            return self.listing(env, start_response)
         elif env['PATH_INFO'] == '/v1/a/c3b/index.html':
             resp = Response(status='204 No Content')
             resp.app_iter = iter([])
@@ -133,12 +166,7 @@ class FakeApp(object):
             return Response(status='404 Not Found')(env, start_response)
         elif env['PATH_INFO'] == '/v1/a/c4':
             self.get_c4_called = True
-            return self.listing(env, start_response,
-                          {'x-container-read': '.r:*',
-                           'x-container-meta-web-index': 'index.html',
-                           'x-container-meta-web-error': 'error.html',
-                           'x-container-meta-web-listings': 't',
-                           'x-container-meta-web-listings-css': 'listing.css'})
+            return self.listing(env, start_response)
         elif env['PATH_INFO'] == '/v1/a/c4/one.txt':
             return Response(status='200 Ok',
                 headers={'x-object-meta-test': 'value'},
@@ -161,15 +189,11 @@ class FakeApp(object):
 <html>
     <body style="background: #000000; color: #ffaaaa">
         <p>Chrome's 404 fancy-page sucks.</p>
-    <body>
-<html>
+    </body>
+</html>
             '''.strip())(env, start_response)
         elif env['PATH_INFO'] == '/v1/a/c5':
-            return self.listing(env, start_response,
-                                {'x-container-read': '.r:*',
-                                 'x-container-meta-web-index': 'index.html',
-                                 'x-container-meta-listings': 't',
-                                 'x-container-meta-web-error': 'error.html'})
+            return self.listing(env, start_response)
         elif env['PATH_INFO'] == '/v1/a/c5/index.html':
             return Response(status='503 Service Unavailable')(env,
                                                               start_response)
@@ -182,45 +206,65 @@ class FakeApp(object):
         elif env['PATH_INFO'] == '/v1/a/c5/404error.html':
             return Response(status='404 Not Found')(env, start_response)
         elif env['PATH_INFO'] == '/v1/a/c6':
-            return self.listing(env, start_response,
-                                {'x-container-read': '.r:*',
-                                 'x-container-meta-web-listings': 't'})
+            return self.listing(env, start_response)
         elif env['PATH_INFO'] == '/v1/a/c6/subdir':
             return Response(status='404 Not Found')(env, start_response)
         elif env['PATH_INFO'] in ('/v1/a/c7', '/v1/a/c7/'):
-            return self.listing(env, start_response,
-                                {'x-container-read': '.r:*',
-                                 'x-container-meta-web-listings': 'f'})
+            return self.listing(env, start_response)
         elif env['PATH_INFO'] in ('/v1/a/c8', '/v1/a/c8/'):
-            return self.listing(env, start_response,
-                          {'x-container-read': '.r:*',
-                           'x-container-meta-web-error': 'error.html',
-                           'x-container-meta-web-listings': 't',
-                           'x-container-meta-web-listings-css': \
-                               'http://localhost/stylesheets/listing.css'})
+            return self.listing(env, start_response)
         elif env['PATH_INFO'] == '/v1/a/c8/subdir/':
             return Response(status='404 Not Found')(env, start_response)
         elif env['PATH_INFO'] in ('/v1/a/c9', '/v1/a/c9/'):
-            return self.listing(env, start_response,
-                          {'x-container-read': '.r:*',
-                           'x-container-meta-web-error': 'error.html',
-                           'x-container-meta-web-listings': 't',
-                           'x-container-meta-web-listings-css': \
-                               '/absolute/listing.css'})
+            return self.listing(env, start_response)
         elif env['PATH_INFO'] == '/v1/a/c9/subdir/':
             return Response(status='404 Not Found')(env, start_response)
         elif env['PATH_INFO'] in ('/v1/a/c10', '/v1/a/c10/'):
-            return self.listing(env, start_response,
-                          {'x-container-read': '.r:*',
-                           'x-container-meta-web-listings': 't'})
+            return self.listing(env, start_response)
         elif env['PATH_INFO'] == '/v1/a/c10/\xe2\x98\x83/':
             return Response(status='404 Not Found')(env, start_response)
         elif env['PATH_INFO'] == '/v1/a/c10/\xe2\x98\x83/\xe2\x98\x83/':
             return Response(status='404 Not Found')(env, start_response)
+        elif env['PATH_INFO'] in ('/v1/a/c11', '/v1/a/c11/'):
+            return self.listing(env, start_response)
+        elif env['PATH_INFO'] == '/v1/a/c11/subdir/':
+            return Response(status='200 Ok', headers={'Content-Type':\
+                            'application/directory'})(env, start_response)
+        elif env['PATH_INFO'] == '/v1/a/c11/subdir/index.html':
+            return Response(status='200 Ok', body='''
+<html>
+    <body>
+        <h2>c11 subdir index</h2>
+    </body>
+</html>
+            '''.strip())(env, start_response)
+        elif env['PATH_INFO'] == '/v1/a/c11/subdir2/':
+            return Response(status='200 Ok', headers={'Content-Type':\
+                            'application/directory'})(env, start_response)
+        elif env['PATH_INFO'] == '/v1/a/c11/subdir2/index.html':
+            return Response(status='404 Not Found')(env, start_response)
+        elif env['PATH_INFO'] in ('/v1/a/c11a', '/v1/a/c11a/'):
+            return self.listing(env, start_response)
+        elif env['PATH_INFO'] == '/v1/a/c11a/subdir/':
+            return Response(status='200 Ok', headers={'Content-Type':\
+                            'text/directory'})(env, start_response)
+        elif env['PATH_INFO'] == '/v1/a/c11a/subdir/index.html':
+            return Response(status='404 Not Found')(env, start_response)
+        elif env['PATH_INFO'] == '/v1/a/c11a/subdir2/':
+            return Response(status='200 Ok', headers={'Content-Type':\
+                            'application/directory'})(env, start_response)
+        elif env['PATH_INFO'] == '/v1/a/c11a/subdir2/index.html':
+            return Response(status='404 Not Found')(env, start_response)
+        elif env['PATH_INFO'] == '/v1/a/c11a/subdir3/':
+            return Response(status='200 Ok', headers={'Content-Type':\
+                            'not_a/directory'})(env, start_response)
+        elif env['PATH_INFO'] == '/v1/a/c11a/subdir3/index.html':
+            return Response(status='404 Not Found')(env, start_response)
         else:
             raise Exception('Unknown path %r' % env['PATH_INFO'])
 
-    def listing(self, env, start_response, headers):
+    def listing(self, env, start_response):
+        headers = {'x-container-read':  '.r:*'}
         if env['PATH_INFO'] in ('/v1/a/c3', '/v1/a/c4', '/v1/a/c8', \
                '/v1/a/c9') and \
                env['QUERY_STRING'] == 'delimiter=/&format=json&prefix=subdir/':
@@ -358,6 +402,11 @@ class TestStaticWeb(unittest.TestCase):
     def setUp(self):
         self.app = FakeApp()
         self.test_staticweb = staticweb.filter_factory({})(self.app)
+        self._orig_get_container_info = staticweb.get_container_info
+        staticweb.get_container_info = mock_get_container_info
+
+    def tearDown(self):
+        staticweb.get_container_info = self._orig_get_container_info
 
     def test_app_set(self):
         app = FakeApp()
@@ -460,11 +509,6 @@ class TestStaticWeb(unittest.TestCase):
         self.assertEquals(resp.status_int, 200)
         self.assert_('Test main index.html file.' in resp.body)
 
-    def test_container3subdir(self):
-        resp = Request.blank(
-                '/v1/a/c3/subdir').get_response(self.test_staticweb)
-        self.assertEquals(resp.status_int, 301)
-
     def test_container3subsubdir(self):
         resp = Request.blank(
                 '/v1/a/c3/subdir3/subsubdir').get_response(self.test_staticweb)
@@ -533,44 +577,6 @@ class TestStaticWeb(unittest.TestCase):
         self.assertEquals(resp.status_int, 404)
         self.assert_("Chrome's 404 fancy-page sucks." in resp.body)
 
-    def test_container4unknown_memcache(self):
-        fake_memcache = FakeMemcache()
-        self.assertEquals(fake_memcache.store, {})
-        resp = Request.blank('/v1/a/c4',
-            environ={'swift.cache': fake_memcache}
-        ).get_response(self.test_staticweb)
-        self.assertEquals(resp.status_int, 301)
-        self.assertEquals(fake_memcache.store,
-           {'/staticweb/v1/a/c4':
-                ('index.html', 'error.html', 't', 'listing.css')})
-        self.assert_(self.test_staticweb.app.get_c4_called)
-        self.test_staticweb.app.get_c4_called = False
-        resp = Request.blank('/v1/a/c4',
-            environ={'swift.cache': fake_memcache}
-        ).get_response(self.test_staticweb)
-        self.assertEquals(resp.status_int, 301)
-        self.assert_(not self.test_staticweb.app.get_c4_called)
-        self.assertEquals(fake_memcache.store,
-           {'/staticweb/v1/a/c4':
-                ('index.html', 'error.html', 't', 'listing.css')})
-        resp = Request.blank('/v1/a/c4',
-            environ={'swift.cache': fake_memcache, 'REQUEST_METHOD': 'PUT'}
-        ).get_response(self.test_staticweb)
-        self.assertEquals(resp.status_int, 200)
-        self.assertEquals(fake_memcache.store, {})
-        resp = Request.blank('/v1/a/c4',
-            environ={'swift.cache': fake_memcache}
-        ).get_response(self.test_staticweb)
-        self.assertEquals(resp.status_int, 301)
-        self.assertEquals(fake_memcache.store,
-           {'/staticweb/v1/a/c4':
-                ('index.html', 'error.html', 't', 'listing.css')})
-        resp = Request.blank('/v1/a/c4',
-            environ={'swift.cache': fake_memcache, 'REQUEST_METHOD': 'POST'}
-        ).get_response(self.test_staticweb)
-        self.assertEquals(resp.status_int, 200)
-        self.assertEquals(fake_memcache.store, {})
-
     def test_container4subdir(self):
         resp = Request.blank(
                 '/v1/a/c4/subdir/').get_response(self.test_staticweb)
@@ -661,6 +667,27 @@ class TestStaticWeb(unittest.TestCase):
         self.assert_(
                 'Listing of /v1/a/c10/\xe2\x98\x83/\xe2\x98\x83/' in resp.body)
 
+    def test_container11subdirmarkerobjectindex(self):
+        resp = Request.blank('/v1/a/c11/subdir/').get_response(
+                self.test_staticweb)
+        self.assertEquals(resp.status_int, 200)
+        self.assert_('<h2>c11 subdir index</h2>' in resp.body)
+
+    def test_container11subdirmarkermatchdirtype(self):
+        resp = Request.blank('/v1/a/c11a/subdir/').get_response(
+                self.test_staticweb)
+        self.assertEquals(resp.status_int, 404)
+
+    def test_container11subdirmarkeraltdirtype(self):
+        resp = Request.blank('/v1/a/c11a/subdir2/').get_response(
+                self.test_staticweb)
+        self.assertEquals(resp.status_int, 200)
+
+    def test_container11subdirmarkerinvaliddirtype(self):
+        resp = Request.blank('/v1/a/c11a/subdir3/').get_response(
+                self.test_staticweb)
+        self.assertEquals(resp.status_int, 200)
+
     def test_subrequest_once_if_possible(self):
         resp = Request.blank(
                 '/v1/a/c4/one.txt').get_response(self.test_staticweb)
