@@ -16,6 +16,7 @@
 import unittest
 from contextlib import contextmanager
 from base64 import b64encode
+from time import time
 
 from swift.common.middleware import tempauth as auth
 from swift.common.swob import Request, Response
@@ -29,11 +30,11 @@ class FakeMemcache(object):
     def get(self, key):
         return self.store.get(key)
 
-    def set(self, key, value, timeout=0):
+    def set(self, key, value, time=0):
         self.store[key] = value
         return True
 
-    def incr(self, key, timeout=0):
+    def incr(self, key, time=0):
         self.store[key] = self.store.setdefault(key, 0) + 1
         return self.store[key]
 
@@ -125,6 +126,8 @@ class TestAuth(unittest.TestCase):
         self.assertEquals(ath.auth_prefix, '/auth/')
         ath = auth.filter_factory({'auth_prefix': ''})(app)
         self.assertEquals(ath.auth_prefix, '/auth/')
+        ath = auth.filter_factory({'auth_prefix': '/'})(app)
+        self.assertEquals(ath.auth_prefix, '/auth/')
         ath = auth.filter_factory({'auth_prefix': '/test/'})(app)
         self.assertEquals(ath.auth_prefix, '/test/')
         ath = auth.filter_factory({'auth_prefix': '/test'})(app)
@@ -325,6 +328,15 @@ class TestAuth(unittest.TestCase):
         req.acl = '.r:.example.com,.rlistings'
         self.assertEquals(self.test_auth.authorize(req), None)
 
+    def test_detect_reseller_request(self):
+        req = self._make_request('/v1/AUTH_admin',
+                                 headers={'X-Auth-Token': 'AUTH_t'})
+        cache_key = 'AUTH_/token/AUTH_t'
+        cache_entry = (time()+3600, '.reseller_admin')
+        req.environ['swift.cache'].set(cache_key, cache_entry)
+        req.get_response(self.test_auth)
+        self.assertTrue(req.environ.get('reseller_request', False))
+
     def test_account_put_permissions(self):
         req = self._make_request('/v1/AUTH_new',
                                  environ={'REQUEST_METHOD': 'PUT'})
